@@ -24,7 +24,7 @@
 /* FIXME: Should use BonoboUIHandler.  */
 
 #include <config.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <gnome.h>
 #include <bonobo.h>
 
