@@ -238,12 +238,6 @@ typedef enum {
 	GTK_HTML_EDITOR_EVENT_DELETE
 } GtkHTMLEditorEventType;
 
-typedef enum {
-	GTK_HTML_ETCH_IN,
-	GTK_HTML_ETCH_OUT,
-	GTK_HTML_ETCH_NONE
-} GtkHTMLEtchStyle; 
-
 typedef enum {
 	GTK_HTML_FONT_STYLE_SHIFT_FIRST = 3,
 	GTK_HTML_FONT_STYLE_SHIFT_BOLD = GTK_HTML_FONT_STYLE_SHIFT_FIRST,
