@@ -100,6 +100,21 @@ clueflow_style_to_string (HTMLClueFlowStyle style)
 	}
 }
 
+static char *
+direction_to_string (HTMLDirection dir)
+{
+	switch (dir) {
+	case HTML_DIRECTION_DERIVED:
+		return "derived";
+	case HTML_DIRECTION_RTL:
+		return "rtl";
+	case HTML_DIRECTION_LTR:
+		return "ltr";
+	default:
+		return "UNKNOWN";
+	}
+}
+
 
 void
 gtk_html_debug_dump_table (HTMLObject *o,
@@ -138,9 +153,10 @@ gtk_html_debug_dump_object (HTMLObject *obj,
 	for (i = 0; i < level; i++)
 		g_print (" ");
 
-	g_print ("ObjectType: %s Pos: %d, %d, MinWidth: %d, Width: %d PrefWidth: %d MaxWidth: %d Ascent %d Descent %d",
+	g_print ("ObjectType: %s Pos: %d, %d, MinWidth: %d, Width: %d PrefWidth: %d MaxWidth: %d Ascent %d Descent %d Direction %s",
 		 html_type_name (HTML_OBJECT_TYPE (obj)),
-		 obj->x, obj->y, obj->min_width, obj->width, obj->pref_width, obj->max_width, obj->ascent, obj->descent);
+		 obj->x, obj->y, obj->min_width, obj->width, obj->pref_width, obj->max_width, obj->ascent, obj->descent,
+		 direction_to_string (html_object_get_direction (obj)));
 
 	if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_CLUEFLOW) {
 		g_print (" [%s, %d]",
@@ -154,6 +170,9 @@ gtk_html_debug_dump_object (HTMLObject *obj,
 		g_print ("]");
 	}
 	else if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_TEXTSLAVE) {
+		GSList *cur;
+		int width;
+		int ii, io, tw, offset;
 		gchar *sl_text = g_strndup (html_text_get_text (HTML_TEXT (HTML_TEXT_SLAVE (obj)->owner),
 								HTML_TEXT_SLAVE (obj)->posStart),
 					    html_text_get_index (HTML_TEXT (HTML_TEXT_SLAVE (obj)->owner),
@@ -161,11 +180,38 @@ gtk_html_debug_dump_object (HTMLObject *obj,
 								 + HTML_TEXT_SLAVE (obj)->posLen)
 					    - html_text_get_index (HTML_TEXT (HTML_TEXT_SLAVE (obj)->owner),
 								   HTML_TEXT_SLAVE (obj)->posStart));
-		g_print ("[start %d end %d] \"%s\" ",
+		g_print ("[start %d end %d len %d] \"%s\"\n",
 			 HTML_TEXT_SLAVE (obj)->posStart,
 			 HTML_TEXT_SLAVE (obj)->posStart + HTML_TEXT_SLAVE (obj)->posLen - 1,
+			 HTML_TEXT_SLAVE (obj)->posLen,
 			 sl_text);
 		g_free (sl_text);
+
+		width = 0;
+		for (cur = HTML_TEXT_SLAVE (obj)->glyph_items; cur; cur = cur->next) {
+			HTMLTextSlaveGlyphItem *sgi = (HTMLTextSlaveGlyphItem *) cur->data;
+			PangoItem *item = sgi->glyph_item.item;
+			PangoRectangle log_rect;
+
+			pango_glyph_string_extents (sgi->glyph_item.glyphs, sgi->glyph_item.item->analysis.font, NULL, &log_rect);
+			g_print ("type: %d item level: %d offset: %d length: %d width: %d chars %d glyphs %d\n", sgi->type, item->analysis.level, item->offset, item->length, log_rect.width, item->num_chars, sgi->glyph_item.glyphs->num_glyphs);
+			width += log_rect.width;
+
+		}
+
+		g_print ("total width by glyph string extents (in engine units): %d\n", (int)(0.5 + width / PANGO_SCALE));
+
+		ii = html_text_get_item_index (HTML_TEXT_SLAVE (obj)->owner, NULL, HTML_TEXT_SLAVE (obj)->posStart, &io);
+		tw = 0;
+		offset = 0;
+		while (offset < HTML_TEXT_SLAVE (obj)->posLen) {
+			tw += HTML_TEXT_SLAVE (obj)->owner->pi->entries [ii].widths [io];
+			g_print ("%d ", HTML_TEXT_SLAVE (obj)->owner->pi->entries [ii].widths [io]);
+			html_text_pi_forward (HTML_TEXT_SLAVE (obj)->owner->pi, &ii, &io);
+			offset ++;
+		}
+
+		g_print ("\ntotal width by widths in entries (in engine units): %d", (int)(0.5 + tw / PANGO_SCALE));
 	}
 
 	g_print ("\n");
@@ -175,15 +221,52 @@ gtk_html_debug_dump_object (HTMLObject *obj,
 		gtk_html_debug_dump_table (obj, level + 1);
 		break;
 	case HTML_TYPE_TEXT:
-	case HTML_TYPE_LINKTEXT:
+	case HTML_TYPE_LINKTEXT: {
+		HTMLText *text = HTML_TEXT (obj);
 		for (i = 0; i < level; i++)
 			g_print (" ");
-		g_print ("Text (%d): \"%s\"\n",
-			 HTML_TEXT (obj)->text_len, HTML_TEXT (obj)->text);
+		g_print ("Text (len %d bytes %d): \"%s\"\n",
+			 text->text_len, text->text_bytes, text->text);
+/* 		debug_spell_errors (text->spell_errors); */
+		if (text->pi) {
+			for (i =0; i < text->pi->n; i ++)
+				g_print ("item %d offset: %d length: %d\n", i, text->pi->entries [i].glyph_item.item->offset, text->pi->entries [i].glyph_item.item->length);
+				
+			for (i = 0; i < text->text_len; i ++) {
+				union {
+					PangoLogAttr attr;
+					guint as_int;
+				} u;
+				u.attr = text->pi->attrs [i];
+				g_print ("log attrs[%d]: %d\n\t", i, u.as_int & 0x7ff);
+				if (u.attr.is_line_break)
+					g_print ("line break, ");
+				if (u.attr.is_mandatory_break)
+					g_print ("mandatory break, ");
+				if (u.attr.is_char_break)
+					g_print ("char break, ");
+				if (u.attr.is_white)
+					g_print ("white, ");
+				if (u.attr.is_cursor_position)
+					g_print ("cursor position, ");
+				if (u.attr.is_word_start)
+					g_print ("word start, ");
+				if (u.attr.is_word_end)
+					g_print ("word end, ");
+				if (u.attr.is_sentence_boundary)
+					g_print ("sentence boundary, ");
+				if (u.attr.is_sentence_start)
+					g_print ("sentence start, ");
+				if (u.attr.is_sentence_end)
+					g_print ("sentence end, ");
+				g_print ("\n");
+			}
+		}
 		break;
-
-	case HTML_TYPE_CLUEH:
+	}
 	case HTML_TYPE_CLUEV:
+		g_print ("Direction: %s\n", direction_to_string (HTML_CLUEV (obj)->dir));
+	case HTML_TYPE_CLUEH:
 	case HTML_TYPE_CLUEFLOW:
 		/* g_print ("Head: %p Tail: %p\n", HTML_CLUE (obj)->head, HTML_CLUE (obj)->tail); */
 	case HTML_TYPE_CLUEALIGNED:
@@ -257,25 +340,6 @@ dump_object_simple (HTMLObject *obj,
 		g_print ("len %d bytes %d\n", text->text_len, text->text_bytes);
 		gtk_html_debug_list_links (text);
 		gtk_html_debug_list_text_attrs (text);
-		/* debug_spell_errors (text->spell_errors); */
-		if (text->pi) {
-			for (i =0; i < text->pi->n; i ++)
-				g_print ("item %d offset: %d length: %d\n", i, text->pi->entries [i].item->offset, text->pi->entries [i].item->length);
-				
-			for (i = 0; i < text->text_len; i ++) {
-				PangoLogAttr attr = text->pi->attrs [i];
-				g_print ("log attrs[%d]: %d\n\t", i, (*(int *)&attr) & 0x7ff);
-				if (attr.is_line_break)
-					g_print ("line break, ");
-				if (attr.is_mandatory_break)
-					g_print ("mandatory break, ");
-				if (attr.is_char_break)
-					g_print ("char break, ");
-				if (attr.is_white)
-					g_print ("white, ");
-				g_print ("\n");
-			}
-		}
 	} else if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_TEXTSLAVE) {
 		HTMLTextSlave *slave = HTML_TEXT_SLAVE (obj);
 		gchar *text;
@@ -360,7 +424,7 @@ gtk_html_debug_dump_list_simple (GList *list,
 #define D_ATTR_TYPE(x, s) if ((attr = pango_attr_iterator_get (iter, PANGO_ATTR_ ## x))) g_print ("%3d-%3d: %s\n", attr->start_index, attr->end_index, s);
 #define D_ATTR_TYPE_INT(x, s) if ((attr = pango_attr_iterator_get (iter, PANGO_ATTR_ ## x))) { g_print ("%3d-%3d: %s %d\n", attr->start_index, attr->end_index, s, ((PangoAttrInt *)attr)->value); }
 
-void
+static void
 gtk_html_debug_list_attrs (PangoAttrList *attrs)
 {
 	PangoAttrIterator *iter = pango_attr_list_get_iterator (attrs);
