@@ -26,10 +26,10 @@
 #include <gnome.h>
 #include <ctype.h>
 #include <libgnomeprint/gnome-print.h>
+#include <libgnomeprint/gnome-print-pango.h>
 #include <libgnomeprint/gnome-print-paper.h>
 
 #include "htmlembedded.h"
-#include "htmlentity.h"
 #include "gtkhtml-embedded.h"
 #include "htmlfontmanager.h"
 #include "htmlprinter.h"
@@ -551,232 +551,104 @@ fill_rect (HTMLPainter *painter, gint x, gint y, gint width, gint height)
 }
 
 static void
-process_attrs (HTMLPrinter *printer, GSList *attrs, GtkHTMLFontStyle *style, gboolean *bgcolor, gboolean *underline, gboolean *strikethrough, gboolean set_context)
+draw_lines (HTMLPrinter *printer, double x, double y, double width, PangoAnalysis *analysis, HTMLPangoProperties *properties)
 {
-	gboolean set_color = FALSE;
+	PangoFontMetrics *metrics;
+	
+	if (!properties->underline && !properties->strikethrough)
+		return;
+	
+	metrics = pango_font_get_metrics (analysis->font, analysis->language);
 
-	while (attrs) {
-		PangoAttribute *attr = attrs->data;
+	gnome_print_setlinecap (printer->context, GDK_CAP_BUTT);
 
-		switch (attr->klass->type) {
-		case PANGO_ATTR_FOREGROUND: {
-			PangoColor pc;
-      
-			if (set_context) {
-				pc = ((PangoAttrColor *) attr)->color;
-				gnome_print_setrgbcolor (printer->context,
-							 pc.red / 65535.0, pc.green / 65535.0, pc.blue / 65535.0);
-				set_color = TRUE;
-			}
-		}
-		break;
-		case PANGO_ATTR_BACKGROUND:
-			if (bgcolor)
-				*bgcolor = TRUE;
-		case PANGO_ATTR_WEIGHT:
-			if (style)
-				*style |= GTK_HTML_FONT_STYLE_BOLD;
-			break;
-		case PANGO_ATTR_STYLE:
-			if (style)
-				*style |= GTK_HTML_FONT_STYLE_ITALIC;
-			break;
-		case PANGO_ATTR_UNDERLINE:
-			if (underline)
-				*underline = TRUE;
-			break;
-		case PANGO_ATTR_STRIKETHROUGH:
-			if (strikethrough)
-				*strikethrough = TRUE;
-			break;
-		case PANGO_ATTR_SIZE:
-			if (style)
-				*style |= ((HTMLPangoAttrFontSize *) attr)->style;
-			break;
-		case PANGO_ATTR_FAMILY: {
-			PangoAttrString *as = (PangoAttrString *) attr;
+	if (properties->underline) {
+#ifdef PANGO_1_5_OR_HIGHER
+		double thickness = SCALE_PANGO_TO_GNOME_PRINT (pango_font_metrics_get_underline_thickness (metrics));
+		double position = SCALE_PANGO_TO_GNOME_PRINT (pango_font_metrics_get_underline_position (metrics));
+		double ly = y + position - thickness / 2;
+#else
+		double thickness = 1.0;
+		double position = -1.0;
+		double ly = y + position - thickness / 2;
+#endif
 
-			if (!strcmp (as->value, "Monospace")
-			    || (HTML_PAINTER (printer)->font_manager.fixed.face && !strcmp (as->value, HTML_PAINTER (printer)->font_manager.fixed.face)))
-				*style |= GTK_HTML_FONT_STYLE_FIXED;
-		}
-		break;
-		default:
-			break;
-		}
-		attrs = attrs->next;
+		gnome_print_newpath (printer->context);
+		gnome_print_moveto (printer->context, x, ly);
+		gnome_print_lineto (printer->context, x + width, ly);
+		gnome_print_setlinewidth (printer->context, thickness);
+		gnome_print_stroke (printer->context);
 	}
-
-	if (set_context && !set_color)
-		gnome_print_setrgbcolor (printer->context, 0.0, 0.0, 0.0);
-}
-
-static void
-show_sized_with_nbsps_replaced (GnomePrintContext *pc, const char *text, int bytes)
-{
-	const char *cur_text;
-	char *cur_replaced, *match, *replaced, *to_free = NULL;
-	int replaced_bytes;
-
-	if (bytes < 8192)
-		replaced = alloca (bytes);
-	else
-		to_free = replaced = g_malloc (bytes);
-
-	cur_text = text;
-	cur_replaced = replaced;
-	replaced_bytes = bytes;
-	while (cur_text < text + bytes) {
-		/* look for begining of 0xc2 0xa0 sequence - utf8 nbsp; */
-		match = memchr (cur_text, 0xc2, bytes - (cur_text - text));
-		if (match != NULL && match - text < bytes && IS_UTF8_NBSP ((guchar *)match)) {
-			strncpy (cur_replaced, cur_text, match - cur_text);
-			cur_replaced += match - cur_text;
-			*cur_replaced = ' ';
-			cur_replaced ++;
-			cur_text = match + 2;
-			replaced_bytes --;
-		} else {
-			strncpy (cur_replaced, cur_text, bytes - (cur_text - text));
-			break;
-		}
+		
+	if (properties->strikethrough) {
+#ifdef PANGO_1_5_OR_HIGHER
+		double thickness = SCALE_PANGO_TO_GNOME_PRINT (pango_font_metrics_get_strikethrough_thickness (metrics));
+		double position = SCALE_PANGO_TO_GNOME_PRINT (pango_font_metrics_get_strikethrough_position (metrics));
+		double ly = y + position - thickness / 2;
+#else
+		double thickness = 1.0;
+		double position = SCALE_PANGO_TO_GNOME_PRINT (pango_font_metrics_get_ascent (metrics)/3);
+		double ly = y + position - thickness / 2;
+#endif
+		gnome_print_newpath (printer->context);
+		gnome_print_moveto (printer->context, x, ly);
+		gnome_print_lineto (printer->context, x + width, ly);
+		gnome_print_setlinewidth (printer->context, thickness);
+		gnome_print_stroke (printer->context);
 	}
-
-	replaced [replaced_bytes] = 0;
-	gnome_print_show_sized (pc, replaced, replaced_bytes);
-
-	g_free (to_free);
 }
 
 static gint
-draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, HTMLTextPangoInfo *pi, PangoAttrList *attrs, GList *glyphs, gint start_byte_offset)
+draw_glyphs (HTMLPainter *painter, gint x, gint y, PangoItem *item, PangoGlyphString *glyphs, GdkColor *fg, GdkColor *bg)
 {
-	GnomeFont *font;
 	HTMLPrinter *printer;
 	gdouble print_x, print_y;
-	double width, asc, dsc;
-	gboolean temp_pi = FALSE;
-
-	width = 0.0;
+	PangoRectangle log_rect;
+	HTMLPangoProperties properties;
 
 	printer = HTML_PRINTER (painter);
-	g_return_val_if_fail (printer->context != NULL, 0);
 
-	if (!pi) {
-		pi = html_painter_text_itemize_and_prepare_glyphs (painter, html_painter_get_font (painter, painter->font_face, painter->font_style),
-								   text, g_utf8_offset_to_pointer (text, len) - text, &glyphs, attrs);
-		start_byte_offset = 0;
-		temp_pi = TRUE;
+	html_printer_coordinates_to_gnome_print (printer, x, y, &print_x, &print_y);
+
+	gnome_print_gsave (printer->context);
+
+	html_pango_get_item_properties (item, &properties);
+	
+	pango_glyph_string_extents (glyphs, item->analysis.font, NULL, &log_rect);
+	
+	if (properties.bg_color) {
+		gnome_print_setrgbcolor (printer->context,
+					 properties.bg_color->red / 65535.0,
+					 properties.bg_color->green / 65535.0,
+					 properties.bg_color->blue / 65535.0);
+		gnome_print_rect_filled (printer->context,
+					 print_x,
+					 print_y - SCALE_PANGO_TO_GNOME_PRINT (log_rect.y + log_rect.height),
+					 SCALE_PANGO_TO_GNOME_PRINT (log_rect.width),
+					 SCALE_PANGO_TO_GNOME_PRINT (log_rect.height));
 	}
-
-	font = html_painter_get_font (painter, painter->font_face, painter->font_style);
-	dsc = -gnome_font_get_descender (font);
-	asc = gnome_font_get_ascender (font);
-
-	if (pi && pi->n) {
-		GList *gl;
-		PangoGlyphString *str;
-		GtkHTMLFontStyle style;
-		guint char_offset = 0;
-		gint ii;
-		const gchar *c_text = text;
-		PangoAttrIterator *iter = NULL;
-
-		html_printer_coordinates_to_gnome_print (printer, x, y, &print_x, &print_y);
-		c_text = text;
-		for (gl = glyphs; gl && char_offset < len; gl = gl->next) {
-			GnomeFont *c_font;
-			GdkGC *bg_gc;
-			gboolean underline, strikethrough, bgcolor;
-			gint c_bytes, begin, end;
-			double cw = 0.0;
-
-			str = (PangoGlyphString *) gl->data;
-			gl = gl->next;
-			ii = GPOINTER_TO_INT (gl->data);
-			bg_gc = NULL;
-			c_font = font;
-			style = GTK_HTML_FONT_STYLE_DEFAULT;
-			c_bytes = g_utf8_offset_to_pointer (c_text, str->num_glyphs) - c_text;
-			gnome_print_gsave (printer->context);
-			bgcolor = underline = strikethrough = FALSE;
-
-			if (attrs)
-				iter = pango_attr_list_get_iterator (attrs);
-
-			if (iter) {
-				do {
-					pango_attr_iterator_range (iter, &begin, &end);
-					if (iter && MAX (begin, c_text - text) < MIN (end, (c_text - text) + c_bytes)) {
-						GSList *attr_list;
-						attr_list = pango_attr_iterator_get_attrs (iter);
-						process_attrs (printer, attr_list, &style, &bgcolor, &underline, &strikethrough, TRUE);
-						html_text_free_attrs (attr_list);
-					}
-				} while (pango_attr_iterator_next (iter));
-				pango_attr_iterator_destroy (iter);
-			}
-			process_attrs (printer, pi->entries [ii].item->analysis.extra_attrs, &style, &bgcolor, &underline, &strikethrough, TRUE);
-			if (style != GTK_HTML_FONT_STYLE_DEFAULT)
-				c_font = html_painter_get_font (HTML_PAINTER (printer), HTML_PAINTER (printer)->font_face, style);
-
-			/* FIXME if (bgcolor) {
-				PangoRectangle log_rect;
-
-				pango_glyph_string_extents (str, pi->entries [ii].item->analysis.font, NULL, &log_rect);
-				gdk_draw_rectangle (gdk_painter->pixmap, bg_gc, TRUE, x + width, y - PANGO_PIXELS (PANGO_ASCENT (log_rect)),
-						    PANGO_PIXELS (log_rect.width), PANGO_PIXELS (log_rect.height));
-				gdk_gc_unref (bg_gc);
-				} */
-
-			gnome_print_newpath (printer->context);
-			gnome_print_moveto (printer->context, print_x, print_y);
-			gnome_print_setfont (printer->context, c_font);
-			show_sized_with_nbsps_replaced (printer->context, c_text, c_bytes);
-
-			cw = gnome_font_get_width_utf8_sized (c_font, c_text, c_bytes);
-			if (strikethrough || underline) {
-				gint ly;
-
-				gnome_print_setlinewidth (printer->context, 1.0);
-				gnome_print_setlinecap (printer->context, GDK_CAP_BUTT);
-
-				if (underline) {
-					ly = print_y + gnome_font_get_underline_position (c_font);
-					gnome_print_newpath (printer->context);
-					gnome_print_moveto (printer->context, print_x, ly);
-					gnome_print_lineto (printer->context, print_x + cw, ly);
-					gnome_print_setlinewidth (printer->context, gnome_font_get_underline_thickness (c_font));
-					gnome_print_stroke (printer->context);
-				}
-
-				if (strikethrough) {
-					ly = print_y + gnome_font_get_ascender (c_font) / 2.0;
-					gnome_print_newpath (printer->context);
-					gnome_print_moveto (printer->context, print_x, ly);
-					gnome_print_lineto (printer->context, print_x + cw, ly);
-					gnome_print_setlinewidth (printer->context, gnome_font_get_underline_thickness (c_font));
-					gnome_print_stroke (printer->context);
-				}
-			}
-
-			gnome_print_grestore (printer->context);
-
-			width += cw;
-			print_x += cw;
-			c_text += c_bytes;
-			char_offset += str->num_glyphs;
-		}
+	
+	if (properties.fg_color) {
+		gnome_print_setrgbcolor (printer->context,
+					 properties.fg_color->red / 65535.0,
+					 properties.fg_color->green / 65535.0,
+					 properties.fg_color->blue / 65535.0);
+	} else {
+		gnome_print_setrgbcolor (printer->context, 0., 0., 0.);
 	}
-
-	if (temp_pi) {
-		if (glyphs)
-			html_painter_glyphs_destroy (glyphs);
-		if (pi)
-			html_text_pango_info_destroy (pi);
-	}
-
-	return SCALE_GNOME_PRINT_TO_ENGINE (width);
+	
+	gnome_print_moveto (printer->context, print_x, print_y);
+	gnome_print_pango_glyph_string (printer->context, 
+					item->analysis.font,
+					glyphs);
+	draw_lines (printer, print_x, print_y,
+		    SCALE_PANGO_TO_GNOME_PRINT (log_rect.width),
+		    &item->analysis, &properties);
+	
+	gnome_print_grestore (printer->context);
+	
+	
+	return html_painter_pango_to_engine (painter, log_rect.width);
 }
 
 static void
@@ -815,114 +687,6 @@ draw_shade_line (HTMLPainter *painter,
 	/* FIXME */
 }
 
-static void
-text_size (HTMLPainter *painter, const gchar *text, guint blen, HTMLTextPangoInfo *pi, PangoAttrList *attrs, GList *glyphs, gint start_byte_offset,
-	   GnomeFont *font, GtkHTMLFontStyle style, gint *width, gint *asc, gint *dsc)
-{
-	HTMLPrinter *printer;
-	gboolean temp_pi = FALSE;
-	double real_width = 0.0;
-
-	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->context != NULL);
-	g_return_if_fail (font != NULL);
-
-	if (asc)
-		*asc = SCALE_GNOME_PRINT_TO_ENGINE (gnome_font_get_ascender (font));
-	if (dsc)
-		*dsc = SCALE_GNOME_PRINT_TO_ENGINE (-gnome_font_get_descender (font));
-
-	if (!pi) {
-		pi = html_painter_text_itemize_and_prepare_glyphs (painter, html_painter_get_font (painter, painter->font_face, painter->font_style),
-								   text, blen, &glyphs, attrs);
-		start_byte_offset = 0;
-		temp_pi = TRUE;
-	}
-
-	if (pi && pi->n) {
-		GList *gl;
-		guint char_offset = 0;
-		gint ii;
-		const gchar *c_text = text;
-		PangoGlyphString *str;
-		PangoAttrIterator *iter = NULL;
-
-		c_text = text;
-		for (gl = glyphs; gl && char_offset < blen; gl = gl->next) {
-			GnomeFont *c_font;
-			gint c_bytes, begin, end;
-
-			str = (PangoGlyphString *) gl->data;
-			gl = gl->next;
-			ii = GPOINTER_TO_INT (gl->data);
-			style = GTK_HTML_FONT_STYLE_DEFAULT;
-			c_font = font;
-			c_bytes = g_utf8_offset_to_pointer (c_text, str->num_glyphs) - c_text;
-
-			if (attrs)
-				iter = pango_attr_list_get_iterator (attrs);
-
-			if (iter) {
-				do {
-					pango_attr_iterator_range (iter, &begin, &end);
-					if (iter && MAX (begin, c_text - text) < MIN (end, (c_text - text) + c_bytes)) {
-						GSList *attr_list;
-						attr_list = pango_attr_iterator_get_attrs (iter);
-						process_attrs (printer, attr_list, &style, NULL, NULL, NULL, FALSE);
-						html_text_free_attrs (attr_list);
-					}
-				} while (pango_attr_iterator_next (iter));
-				pango_attr_iterator_destroy (iter);
-			}
-
-			process_attrs (printer, pi->entries [ii].item->analysis.extra_attrs, &style, NULL, NULL, NULL, FALSE);
-			if (style != GTK_HTML_FONT_STYLE_DEFAULT)
-				c_font = html_painter_get_font (HTML_PAINTER (printer), HTML_PAINTER (printer)->font_face, style);
-
-			if (asc)
-				*asc = MAX (*asc, SCALE_GNOME_PRINT_TO_ENGINE (gnome_font_get_ascender (c_font)));
-			if (dsc)
-				*dsc = MAX (*dsc, SCALE_GNOME_PRINT_TO_ENGINE (-gnome_font_get_descender (c_font)));
-
-			if (width)
-				real_width += gnome_font_get_width_utf8_sized (c_font, c_text, c_bytes);
-			c_text += c_bytes;
-			char_offset += str->num_glyphs;
-		}
-	}
-
-	if (temp_pi) {
-		if (glyphs)
-			html_painter_glyphs_destroy (glyphs);
-		if (pi)
-			html_text_pango_info_destroy (pi);
-	}
-
-	if (width)
-		*width = SCALE_GNOME_PRINT_TO_ENGINE (real_width);
-}
-
-static void
-calc_text_size_bytes (HTMLPainter *painter, const gchar *text, guint len, HTMLTextPangoInfo *pi, PangoAttrList *attrs, GList *glyphs, gint start_byte_offset,
-		      HTMLFont *font, GtkHTMLFontStyle style, gint *width, gint *asc, gint *dsc)
-{
-	text_size (painter, text, len, pi, attrs, glyphs, start_byte_offset, font->data, style, width, asc, dsc);
-}
-
-static void
-calc_text_size (HTMLPainter *painter, const gchar *text, guint len, HTMLTextPangoInfo *pi, PangoAttrList *attrs, GList *glyphs, gint start_byte_offset,
-		GtkHTMLFontStyle style, HTMLFontFace *face, gint *width, gint *asc, gint *dsc)
-{
-	HTMLPrinter *printer;
-
-	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->context != NULL);
-
-	text_size (painter, text, g_utf8_offset_to_pointer (text, len) - text, pi, attrs, glyphs,
-		   start_byte_offset, html_painter_get_font (painter, face, style), style, width, asc, dsc);
-}
-
-
 static guint
 get_pixel_size (HTMLPainter *painter)
 {
@@ -931,62 +695,6 @@ get_pixel_size (HTMLPainter *painter)
 	return SCALE_GNOME_PRINT_TO_ENGINE (PIXEL_SIZE);
 }
 
-static inline gdouble
-get_font_size (HTMLPrinter *printer, gboolean points, gdouble size)
-{
-	return printer->scale * (points ? PANGO_PIXELS (size) / 10 : PANGO_PIXELS (size));
-}
-
-static HTMLFont *
-alloc_font (HTMLPainter *painter, gchar *face, gdouble size, gboolean points, GtkHTMLFontStyle style)
-{
-	HTMLPrinter *printer = HTML_PRINTER (painter);
-	GnomeFontWeight weight;
-	GnomeFont *font;
-	gboolean italic;
-
-	weight = (style & GTK_HTML_FONT_STYLE_BOLD) ? GNOME_FONT_BOLD : GNOME_FONT_BOOK;
-	italic = (style & GTK_HTML_FONT_STYLE_ITALIC) ? TRUE : FALSE;
-
-	font = gnome_font_find_closest_from_weight_slant (face ? face : (style & GTK_HTML_FONT_STYLE_FIXED ? "Monospace" : "Sans"),
-							  weight, italic, get_font_size (printer, points, size));
-
-	if (font == NULL && face == NULL) {
-		GList *family_list;
-
-		family_list = gnome_font_family_list ();
-		if (family_list && family_list->data) {
-			font = gnome_font_find_closest_from_weight_slant (family_list->data, weight, italic, get_font_size (printer, points, size));
-			gnome_font_family_list_free (family_list);
-		}
-	}
-
-	return font ? html_font_new (font,
-				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, " ", 1)/HTML_PRINTER (printer)->scale),
-				     SCALE_GNOME_PRINT_TO_ENGINE (gnome_font_get_ascender (font)),
-				     SCALE_GNOME_PRINT_TO_ENGINE (-gnome_font_get_descender (font)),
-				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, "\xc2\xa0", 2)/HTML_PRINTER (printer)->scale),
-				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, "\t", 1)/HTML_PRINTER (printer)->scale),
-				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, "e", 1)/HTML_PRINTER (printer)->scale),
-				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, HTML_BLOCK_CITE, strlen (HTML_BLOCK_CITE))
-								       /HTML_PRINTER (printer)->scale),
-				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, HTML_BLOCK_INDENT, strlen (HTML_BLOCK_INDENT))
-								       /HTML_PRINTER (printer)->scale))
-		: NULL;
-}
-
-static void
-ref_font (HTMLPainter *painter, HTMLFont *font)
-{
-	g_object_ref (font->data);
-}
-
-static void
-unref_font (HTMLPainter *painter, HTMLFont *font)
-{
-	g_object_unref (font->data);
-}
-
 static guint
 get_page_width (HTMLPainter *painter, HTMLEngine *e)
 {
@@ -1007,7 +715,8 @@ html_printer_init (GObject *object)
 	printer                = HTML_PRINTER (object);
 	printer->context       = NULL;
 	printer->config        = NULL;
-	printer->scale         = 1.0;
+
+	html_printer_set_scale (printer, 1.0);
 }
 
 static void
@@ -1023,19 +732,14 @@ html_printer_class_init (GObjectClass *object_class)
 
 	painter_class->begin = begin;
 	painter_class->end = end;
-	painter_class->alloc_font = alloc_font;
-	painter_class->ref_font   = ref_font;
-	painter_class->unref_font = unref_font;
 	painter_class->alloc_color = alloc_color;
 	painter_class->free_color = free_color;
-	painter_class->calc_text_size = calc_text_size;
-	painter_class->calc_text_size_bytes = calc_text_size_bytes;
 	painter_class->set_pen = set_pen;
 	painter_class->get_black = get_black;
 	painter_class->draw_line = draw_line;
 	painter_class->draw_rect = draw_rect;
 	painter_class->draw_border = draw_border;
-	painter_class->draw_text = draw_text;
+	painter_class->draw_glyphs = draw_glyphs;
 	painter_class->fill_rect = fill_rect;
 	painter_class->draw_pixmap = draw_pixmap;
 	painter_class->clear = clear;
@@ -1074,15 +778,21 @@ html_printer_get_type (void)
 HTMLPainter *
 html_printer_new (GtkWidget *widget, GnomePrintContext *context, GnomePrintJob *master)
 {
+	PangoFontMap *font_map;
 	HTMLPrinter *new;
+	HTMLPainter *painter;
 
 	new = g_object_new (HTML_TYPE_PRINTER, NULL);
+	painter = HTML_PAINTER (new);
 
 	g_object_ref (context);
 	html_painter_set_widget (HTML_PAINTER (new), widget);
 	new->context = context;
 	new->master = master;
 
+	font_map = gnome_print_pango_get_default_font_map ();
+	painter->pango_context = gnome_print_pango_create_context (font_map);
+	
 	return HTML_PAINTER (new);
 }
 
@@ -1116,3 +826,25 @@ html_printer_get_page_height (HTMLPrinter *printer)
 
 	return engine_height;
 }
+
+/**
+ * html_printer_set_scale:
+ * @printer: a #HTMLPrinter
+ * @scale: new scale factor
+ * 
+ * Sets the scale factor for printer output. Output coordinates
+ * will be multiplied by @scale.
+ **/
+void
+html_printer_set_scale (HTMLPrinter *printer, gdouble scale)
+{
+	HTMLPainter *painter;
+
+	g_return_if_fail (HTML_IS_PRINTER (printer));
+	g_return_if_fail (scale >= 0);
+
+	painter = HTML_PAINTER (printer);
+	
+	printer->scale = scale;
+	painter->engine_to_pango = scale * PANGO_SCALE / 1024.;
+}
